@@ -11,7 +11,7 @@ at the top-level directory.
 
 
 /*! @file 
- * \brief THis file contains the main loop of pdgstrf which involves rank k
+ * \brief This file contains the main loop of pdgstrf which involves rank k
  *        update of the Schur complement.
  *        Uses 2D partitioning for the scatter phase.
  *
@@ -27,29 +27,39 @@ at the top-level directory.
 double tt_start;
 double tt_end;
 if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
-    int cum_nrow=0;
-    int temp_nbrow;
+    int cum_nrow=0; /* cumulative number of nonzero rows in L(:,k) */
+    int temp_nbrow; /* nonzero rows in current block L(i,k) */
     lptr = lptr0;
     luptr = luptr0;
     /**
-     * seperating L blocks
+     * Seperating L blocks into the top part within look-ahead window
+     * and the remaining ones.
      */
      int lookAheadBlk=0, RemainBlk=0;
 
      tt_start = SuperLU_timer_();
 
+     /* Loop through all blocks in L(:,k) to set up pointers to the start 
+      * of each block in the data arrays.
+      *   - lookAheadFullRow[i] := number of nonzero rows from block 0 to i
+      *   - lookAheadStRow[i] := number of nonzero rows before block i
+      *   - lookAhead_lptr[i] := point to the start of block i in L's index[] 
+      *   - (ditto Remain_Info[i])
+      */
      for (int i = 0; i < nlb; ++i) {
-	 ib = lsub[lptr];            /* Row block L(i,k). */
+	 ib = lsub[lptr];            /* block number of L(i,k). */
 	 temp_nbrow = lsub[lptr+1];  /* Number of full rows. */
         
-	 int look_up_flag=1;
+	 int look_up_flag = 1; /* assume ib is outside look-up window */
 	 for (int j = k0+1; j < SUPERLU_MIN (k0 + num_look_aheads+2, nsupers ); ++j)
 	     {
-		 if(ib == perm_c_supno[j]) look_up_flag=0;
+		 if(ib == perm_c_supno[j]) {
+		     look_up_flag=0; /* flag ib is within look-up window */
+                     break; /* Sherry -- can exit the loop?? */
+                 }
 	     }
 	 
-	 if(!look_up_flag) {
-	     /* ib is within look up window */
+	 if( look_up_flag == 0 ) { /* ib is within look up window */
 	     if (lookAheadBlk==0) {
 		 lookAheadFullRow[lookAheadBlk] = temp_nbrow;
 	     } else {
@@ -78,9 +88,9 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
          cum_nrow +=temp_nbrow;
 	 
 	 lptr += LB_DESCRIPTOR;  /* Skip descriptor. */
-	 lptr += temp_nbrow;
+	 lptr += temp_nbrow;     /* Move to next block */
 	 luptr += temp_nbrow;
-     }  /* for i ... */
+     }  /* for i ... all blocks in L(:,k) */
 
      lptr = lptr0;
      luptr = luptr0;
@@ -92,7 +102,10 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
      int LDlookAhead_LBuff = lookAheadBlk==0? 0 :lookAheadFullRow[lookAheadBlk-1];
 #endif
 
-     /* #pragma omp parallel for  */
+     /* Loop through the look-ahead blocks to copy Lval into the buffer */
+#ifdef __OPENMP
+     /* #pragma omp parallel for -- why not?? Sherry */
+#endif
      for (int i = 0; i < lookAheadBlk; ++i) {
 	 int StRowDest  = 0;
 	 int temp_nbrow;
@@ -115,6 +128,8 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
      }
 
      int LDRemain_LBuff = RemainBlk==0 ? 0 : Remain_info[RemainBlk-1].FullRow;
+
+    /* Loop through the remaining blocks to copy Lval into the buffer */
 #ifdef _OPENMP
 #pragma omp parallel for 
 #endif
@@ -146,10 +161,11 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
 #if 0
      LookAheadRowSepMOP  +=  2*knsupc*(lookAheadFullRow[lookAheadBlk-1]+Remain_info[RemainBlk-1].FullRow );
 #else
-     int_t lnbrow, rnbrow;
+     int_t lnbrow, rnbrow; /* number of nonzero rows in look-ahead window
+                              or remaining part.  */
      lnbrow = lookAheadBlk==0 ? 0  : lookAheadFullRow[lookAheadBlk-1];
      rnbrow = RemainBlk==0 ? 0 : Remain_info[RemainBlk-1].FullRow;
-     nbrow = lnbrow + rnbrow;
+     nbrow = lnbrow + rnbrow; /* total number of rows in L */
      LookAheadRowSepMOP += 2*knsupc*(nbrow);
 #endif     
      ldu   =0;
@@ -162,19 +178,25 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
      nbrow = lookAheadFullRow[lookAheadBlk-1]+Remain_info[RemainBlk-1].FullRow;
 #endif
 
-     if ( nbrow>0 ) {
+     if ( nbrow > 0 ) { /* L(:,k) is not empty */
 	 /*
-	  *   counting U blocks
+	  * Counting U blocks
 	  */
-	 ncols=0;
+	 ncols=0; /* total number of nonzero columns in U(k,:) */
 	 ldu=0;
-	 full=1;
+	 full=1; /* flag the U block is indeed 'full', containing segments
+	            of same length. No need padding 0 */
 	 int temp_ncols=0;
-	 for (j = jj0; j < nub; ++j) {
-	     temp_ncols=0;
+
+         /* Loop through all blocks in U(k,:) to set up pointers to the start
+          * of each block in the data arrays, store them in Ublock_info[j]
+          * for block U(k,j).
+  	  */
+	 for (j = jj0; j < nub; ++j) { /* jj0 was set to 0 */
+	     temp_ncols = 0;
 	     arrive_at_ublock(
-			      j,&iukp,&rukp,&jb,&ljb,&nsupc,
-			      iukp0,rukp0,usub,perm_u,xsup,grid
+			      j, &iukp, &rukp, &jb, &ljb, &nsupc,
+			      iukp0, rukp0, usub, perm_u, xsup, grid
 			      );
 	     Ublock_info[j].iukp = iukp;
 	     Ublock_info[j].rukp = rukp;
@@ -187,7 +209,7 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
 		 segsize = klst - usub[jj];
 		 if ( segsize ) {
                     ++temp_ncols;
-                    if ( segsize != ldu ) full = 0;
+                    if ( segsize != ldu ) full = 0; /* need padding 0 */
                     if ( segsize > ldu ) ldu = segsize;
 		 }
 	     }
@@ -196,27 +218,32 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
 	     ncols += temp_ncols;
 	 }
 
-	 /* Now doing prefix sum on  on full_u_cols */
+	 /* Now doing prefix sum on full_u_cols.
+	  * After this, full_u_cols is the number of nonzero columns
+          * from block 0 to block j.
+          */
 	 for ( j = jj0+1; j < nub; ++j) {
 	     Ublock_info[j].full_u_cols += Ublock_info[j-1].full_u_cols;
 	 }
             
-	 tempu = bigU;
+	 tempu = bigU; /* buffer the entire row block U(k,:) */
 
+         /* Gather U(k,:) into buffer bigU[] to prepare for GEMM */
 #ifdef _OPENMP        
 #pragma omp parallel for private(j,iukp,rukp,tempu, jb, nsupc,ljb,segsize,\
 	lead_zero, jj, i) \
         default (shared) schedule(SCHEDULE_STRATEGY)
 #endif
-        for (j = jj0; j < nub; ++j) {
+        for (j = jj0; j < nub; ++j) { /* jj0 was set to 0 */
 
             if(j==jj0) tempu = bigU;
             else tempu = bigU + ldu*Ublock_info[j-1].full_u_cols;
 
             /* == processing each of the remaining columns == */
-            arrive_at_ublock(j,&iukp,&rukp,&jb,&ljb,&nsupc,
-			     iukp0,rukp0,usub,perm_u,xsup,grid);
-            
+            arrive_at_ublock(j, &iukp, &rukp, &jb, &ljb, &nsupc,
+			     iukp0, rukp0, usub,perm_u, xsup, grid);
+
+            /* Copy from U(k,:) to tempu[], padding zeros.  */            
             for (jj = iukp; jj < iukp+nsupc; ++jj) {
                 segsize = klst - usub[jj];
                 if ( segsize ) {
@@ -233,9 +260,9 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
 
         }   /* parallel for j:jjj_st..jjj */
 
-        tempu = bigU;       //setting it to starting of the matrix
+        tempu = bigU;  /* setting to the start of padded U(k,:) */
 
-    }  /* if(nbrow>0) */
+    }  /* end if (nbrow>0) */
 
     tt_end = SuperLU_timer_();
     GatherTimer += tt_end-tt_start;
@@ -244,43 +271,61 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
     int Lnbrow   = lookAheadBlk==0 ? 0 :lookAheadFullRow[lookAheadBlk-1];
     int Rnbrow   = RemainBlk==0 ? 0 : Remain_info[RemainBlk-1].FullRow;
     int jj_cpu=nub;       /*limit between CPU and GPU */
+    int thread_id;
     tempv = bigV;
 
-    if (Lnbrow>0 && ldu >0 && ncols>0) {
-        ncols   = Ublock_info[nub-1].full_u_cols;
-        schur_flop_counter  += 2 * (double)Lnbrow * (double)ldu * (double)ncols;
-        stat->ops[FACT]     += 2 * (double)Lnbrow * (double)ldu * (double)ncols;
+    if ( Lnbrow>0 && ldu>0 && ncols>0 ) { /* Both L(:,k) and U(k,:) nonempty */
+        /* Perform a large GEMM call */
+        ncols = Ublock_info[nub-1].full_u_cols;
+        schur_flop_counter += 2 * (double)Lnbrow * (double)ldu * (double)ncols;
+        stat->ops[FACT]    += 2 * (double)Lnbrow * (double)ldu * (double)ncols;
 
         tt_start = SuperLU_timer_();
 
+        /***************************************************************
+         * Updating look-ahead blocks in both L and U look-ahead windows.
+         ***************************************************************/
 #ifdef _OPENMP
-#pragma omp parallel for default (shared) \
+#pragma omp parallel default (shared) private(thread_id)
+     {
+ 	thread_id = omp_get_thread_num();
+ 
+ 	/* Ideally, should organize the loop as:
+                for (j = 0; j < nub; ++j) {
+                    for (lb = 0; lb < lookAheadBlk; ++lb) {
+ 	               L(lb,k) X U(k,j) -> tempv[]
+                    }
+                }
+ 	   But now, we use collapsed loop to achieve more parallelism.
+ 	   Total number of block updates is:
+ 	      (# of lookAheadBlk in L(:,k)) X (# of blocks in U(k,:))
+ 	*/
+#pragma omp for \
     private (j,i,lb,rukp,iukp,jb,nsupc,ljb,lptr,ib,temp_nbrow,cum_nrow)	\
     schedule(dynamic)
+#else /* not use _OPENMP */
+ 	thread_id = 0;
 #endif
+ 	/* Each thread is assigned one loop index ij, responsible for 
+ 	   block update L(lb,k) * U(k,j) -> tempv[]. */
         for (int ij = 0; ij < lookAheadBlk*(nub-jj0); ++ij) {
-            int j   = ij/lookAheadBlk + jj0; 
+            int j   = ij/lookAheadBlk + jj0; /* jj0 was set to 0 */
             int lb  = ij%lookAheadBlk;
 
-#ifdef _OPENMP            
-            int thread_id = omp_get_thread_num();
-#else
-            int thread_id = 0;
-#endif
             int* indirect_thread    = indirect + ldt*thread_id;
             int* indirect2_thread   = indirect2 + ldt*thread_id;
             double* tempv1 = bigV + thread_id*ldt*ldt; 
 
-            /* Getting U block information */
+            /* Getting U block U(k,j) information */
             /* unsigned long long ut_start, ut_end; */
             int_t rukp =  Ublock_info[j].rukp;
             int_t iukp =  Ublock_info[j].iukp;
             int jb   =  Ublock_info[j].jb;
             int nsupc = SuperSize(jb);
-            int ljb = LBj (jb, grid);
+            int ljb = LBj (jb, grid);  /* destination column block */
             int st_col;
             int ncols;
-            if (j>jj0) {
+            if ( j>jj0 ) { /* jj0 was set to 0 */
                 ncols  = Ublock_info[j].full_u_cols-Ublock_info[j-1].full_u_cols;
                 st_col = Ublock_info[j-1].full_u_cols;
             } else {
@@ -288,7 +333,7 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
                 st_col = 0;   
             }
 
-            /* Getting L block information */
+            /* Getting L block L(i,k) information */
             int_t lptr = lookAhead_lptr[lb];
             int ib   = lookAhead_ib[lb];
             int temp_nbrow = lsub[lptr+1];
@@ -307,34 +352,39 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
             if ( ib < jb ) {
                 dscatter_u (
 				 ib, jb,
-				 nsupc, iukp,xsup,
+				 nsupc, iukp, xsup,
 				 klst, temp_nbrow,
-				 lptr, temp_nbrow,lsub,
+				 lptr, temp_nbrow, lsub,
 				 usub, tempv1,
-				 Ufstnz_br_ptr,
-				 Unzval_br_ptr,
+				 Ufstnz_br_ptr, Unzval_br_ptr,
 				 grid
-				 );
+			        );
             } else {
                 dscatter_l (
-				 ib, ljb, nsupc,iukp,xsup,klst,temp_nbrow,lptr,
-				 temp_nbrow,usub,lsub,tempv1,
+				 ib, ljb, 
+				 nsupc, iukp, xsup,
+ 				 klst, temp_nbrow,
+				 lptr, temp_nbrow,
+				 usub, lsub, tempv1,
 				 indirect_thread, indirect2_thread,
-				 Lrowind_bc_ptr,Lnzval_bc_ptr,grid
-				 );
+				 Lrowind_bc_ptr, Lnzval_bc_ptr,
+				 grid
+				);
             }
-        } /* for ij = ... */
-
+        } /* end omp for ij = ... */
+#ifdef _OPENMP
+    } /* end omp parallel */
+#endif
         tt_end = SuperLU_timer_();
         LookAheadGEMMTimer += tt_end- tt_start;
-        LookAheadGEMMFlOp  += 2 * (double ) Lnbrow * (double )ldu * (double )ncols;
-        stat->ops[FACT]    += 2 * (double ) Lnbrow * (double )ldu * (double )ncols;
+        LookAheadGEMMFlOp  += 2*(double)Lnbrow * (double)ldu * (double)ncols;
+        stat->ops[FACT]    += 2 * (double)Lnbrow * (double)ldu * (double)ncols;
         LookAheadScatterTimer += tt_end-tt_start;
         LookAheadScatterMOP   += 3*Lnbrow*ncols;
-    } /* if Lnbrow < ... */
+    } /* end if Lnbrow < ... */
     
     /***************************************************************
-     *   Updating remaining rows and column on CPU
+     * Updating remaining rows and columns on CPU.
      ***************************************************************/
     Rnbrow  = RemainBlk==0 ? 0 : Remain_info[RemainBlk-1].FullRow;
     ncols   = jj_cpu==0 ? 0 : Ublock_info[jj_cpu-1].full_u_cols;
@@ -345,24 +395,37 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
     tt_start = SuperLU_timer_();
 
 #ifdef _OPENMP
-#pragma omp parallel for default (shared) \
+#pragma omp parallel default(shared) private(thread_id)
+    {
+	thread_id = omp_get_thread_num();
+ 
+	/* Ideally, should organize the loop as:
+               for (j = 0; j < jj_cpu; ++j) {
+                   for (lb = 0; lb < RemainBlk; ++lb) {
+	               L(lb,k) X U(k,j) -> tempv[]
+                   }
+               }
+	   But now, we use collapsed loop to achieve more parallelism.
+	   Total number of block updates is:
+	      (# of RemainBlk in L(:,k)) X (# of blocks in U(k,:))
+	*/
+#pragma omp for \
     private (j,i,lb,rukp,iukp,jb,nsupc,ljb,lptr,ib,temp_nbrow,cum_nrow)	\
     schedule(dynamic)
+#else /* not use _OPENMP */
+    thread_id = 0;
 #endif
-    for (int ij = 0; ij < RemainBlk*(jj_cpu-jj0); ++ij) {
+	/* Each thread is assigned one loop index ij, responsible for 
+	   block update L(lb,k) * U(k,j) -> tempv[]. */
+    for (int ij = 0; ij < RemainBlk*(jj_cpu-jj0); ++ij) { /* jj_cpu := nub */
 	int j   = ij / RemainBlk + jj0; 
 	int lb  = ij % RemainBlk;
 
-#ifdef _OPENMP            
-	int thread_id = omp_get_thread_num();
-#else
-	int thread_id = 0;
-#endif
 	int* indirect_thread = indirect + ldt*thread_id;
 	int* indirect2_thread = indirect2 + ldt*thread_id;
 	double* tempv1 = bigV + thread_id*ldt*ldt; 
 
-	/* Getting U block information */
+	/* Getting U block U(k,j) information */
 	/* unsigned long long ut_start, ut_end; */
 	int_t rukp =  Ublock_info[j].rukp;
 	int_t iukp =  Ublock_info[j].iukp;
@@ -371,7 +434,7 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
 	int ljb = LBj (jb, grid);
 	int st_col;
 	int ncols;
-	if (j>jj0) {
+	if ( j>jj0 ) {
 	    ncols  = Ublock_info[j].full_u_cols-Ublock_info[j-1].full_u_cols;
 	    st_col = Ublock_info[j-1].full_u_cols;
 	} else {
@@ -379,7 +442,7 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
 	    st_col = 0;   
 	}
 
-	/* Getting L block information */
+	/* Getting L block L(i,k) information */
 	int_t lptr = Remain_info[lb].lptr;
 	int ib   = Remain_info[lb].ib;
 	int temp_nbrow = lsub[lptr+1];
@@ -399,24 +462,29 @@ if ( msg0 && msg2 ) {  /* L(:,k) and U(k,:) are not empty. */
 
 	/* Now scattering the block */
 	if ( ib<jb ) {
-	    dscatter_u (
-			     ib, jb,
-			     nsupc, iukp,xsup,
-			     klst, temp_nbrow,
-			     lptr, temp_nbrow,lsub,
-			     usub, tempv1,
-			     Ufstnz_br_ptr,
-			     Unzval_br_ptr,
-			     grid
-			     );
+	    dscatter_u(
+			    ib, jb,
+			    nsupc, iukp, xsup,
+			    klst, temp_nbrow,
+			    lptr, temp_nbrow,lsub,
+			    usub, tempv1,
+			    Ufstnz_br_ptr, Unzval_br_ptr,
+			    grid
+		           );
 	} else {
-	    dscatter_l (
-			     ib, ljb, nsupc,iukp,xsup,klst,temp_nbrow,lptr,
-			     temp_nbrow,usub,lsub,tempv1,
-			     indirect_thread, indirect2_thread,
-			     Lrowind_bc_ptr,Lnzval_bc_ptr,grid
-			     );
+	    dscatter_l(
+			    ib, ljb,
+			    nsupc, iukp, xsup,
+			    klst, temp_nbrow,
+			    lptr, temp_nbrow,
+			    usub, lsub, tempv1,
+			    indirect_thread, indirect2_thread,
+			    Lrowind_bc_ptr,Lnzval_bc_ptr,
+			    grid
+			   );
 	}
-    } /* for (int ij =... */
-        
-}  /* if  k L(:,k) and U(k,:) are not empty */
+    } /* end omp for (int ij =...) */
+#ifdef _OPENMP
+    } /* end omp parallel region */
+#endif
+}  /* end if L(:,k) and U(k,:) are not empty */
