@@ -1,3 +1,13 @@
+/*! \file
+Copyright (c) 2003, The Regents of the University of California, through
+Lawrence Berkeley National Laboratory (subject to receipt of any required 
+approvals from U.S. Dept. of Energy) 
+
+All rights reserved. 
+
+The source code is distributed under BSD license, see the file License.txt
+at the top-level directory.
+*/
 
 /*! @file 
  * \brief Solves a system of distributed linear equations A*X = B with a
@@ -577,8 +587,8 @@ pzgstrs(int_t n, LUstruct_t *LUstruct,
     maxrecvsz = knsupc * nrhs + SUPERLU_MAX( XK_H, LSUM_H );
     if ( !(lsum = doublecomplexCalloc_dist(((size_t)ldalsum)*nrhs + nlb*LSUM_H)) )
 	ABORT("Calloc fails for lsum[].");
-    if ( !(x = doublecomplexMalloc_dist(ldalsum * nrhs + nlb * XK_H)) )
-	ABORT("Malloc fails for x[].");
+    if ( !(x = doublecomplexCalloc_dist(ldalsum * nrhs + nlb * XK_H)) )
+	ABORT("Calloc fails for x[].");
     if ( !(recvbuf = doublecomplexMalloc_dist(maxrecvsz)) )
 	ABORT("Malloc fails for recvbuf[].");
     if ( !(rtemp = doublecomplexCalloc_dist(maxrecvsz)) )
@@ -857,7 +867,7 @@ pzgstrs(int_t n, LUstruct_t *LUstruct,
 
 #if ( DEBUGlevel>=2 )
 	    default:
-	      printf("(%2d) Recv'd wrong message tag %4d\n", status.MPI_TAG);
+	      printf("(%2d) Recv'd wrong message tag %4d\n", iam, status.MPI_TAG);
 	      break;
 #endif
 	  } /* switch */
@@ -1261,7 +1271,7 @@ pzgstrs(int_t n, LUstruct_t *LUstruct,
 
 #if ( DEBUGlevel>=2 )
 	      default:
-		printf("(%2d) Recv'd wrong message tag %4d\n", status.MPI_TAG);
+		printf("(%2d) Recv'd wrong message tag %4d\n", iam, status.MPI_TAG);
 		break;
 #endif		
 
