@@ -1,3 +1,13 @@
+/*! \file
+Copyright (c) 2003, The Regents of the University of California, through
+Lawrence Berkeley National Laboratory (subject to receipt of any required 
+approvals from U.S. Dept. of Energy) 
+
+All rights reserved. 
+
+The source code is distributed under BSD license, see the file License.txt
+at the top-level directory.
+*/
 
 /*! @file 
  * \brief Solves a system of distributed linear equations A*X = B with a general N-by-N matrix A using the LU factorization
@@ -511,7 +521,7 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
 
 #if ( DEBUGlevel>=2 )	      
 	    default:
-	      printf("(%2d) Recv'd wrong message tag %4d\n", status.MPI_TAG);
+	      printf("(%2d) Recv'd wrong message tag %4d\n", iam, status.MPI_TAG);
 	      break;
 #endif
 	  } /* switch */
@@ -917,7 +927,7 @@ pzgstrs_Bglobal(int_t n, LUstruct_t *LUstruct, gridinfo_t *grid,
 
 #if ( DEBUGlevel>=2 )
 	      default:
-		printf("(%2d) Recv'd wrong message tag %4d\n", status.MPI_TAG);
+		printf("(%2d) Recv'd wrong message tag %4d\n", iam, status.MPI_TAG);
 		break;
 #endif		
 
