@@ -1,3 +1,13 @@
+/*! \file
+Copyright (c) 2003, The Regents of the University of California, through
+Lawrence Berkeley National Laboratory (subject to receipt of any required 
+approvals from U.S. Dept. of Energy) 
+
+All rights reserved. 
+
+The source code is distributed under BSD license, see the file License.txt
+at the top-level directory.
+*/
 /*! @file
  * \brief Permutes the columns of the original matrix
  *
@@ -33,7 +43,7 @@ int check_perm_dist(char *, int_t, int_t *);
  * Arguments
  * =========
  *
- * options (input) superlu_options_t*
+ * options (input) superlu_dist_options_t*
  *         Specifies whether or not the elimination tree will be re-used.
  *         If options->Fact == DOFACT, this means first time factor A, 
  *         etree is computed and output.
@@ -67,7 +77,7 @@ int check_perm_dist(char *, int_t, int_t *);
  * </pre>
  */
 void
-sp_colorder(superlu_options_t *options,  SuperMatrix *A, int_t *perm_c, 
+sp_colorder(superlu_dist_options_t *options,  SuperMatrix *A, int_t *perm_c, 
 	    int_t *etree, SuperMatrix *AC)
 {
 
