@@ -67,7 +67,7 @@ at the top-level directory.
   #define mpi_int_t   MPI_SHORT
 #elif defined (_LONGINT)
   typedef long long int int_t;
-  #define mpi_int_t   MPI_LONG
+  #define mpi_int_t   MPI_LONG_LONG_INT
   #define IFMT "%lld"
 #else /* Default */
   typedef int int_t;
@@ -81,6 +81,7 @@ at the top-level directory.
 #include "util_dist.h"
 #include "psymbfact.h"
 
+#define ISORT     /* NOTE: qsort() has bug on Mac */
 
 /*********************************************************************** 
  * Constants
@@ -263,8 +264,8 @@ typedef struct {
 /*-- Process grid definition */
 typedef struct {
     MPI_Comm comm;        /* MPI communicator */
-    superlu_scope_t rscp; /* row scope */
-    superlu_scope_t cscp; /* column scope */
+    superlu_scope_t rscp; /* process scope in rowwise, horizontal directon */
+    superlu_scope_t cscp; /* process scope in columnwise, vertical direction */
     int iam;              /* my process number in this scope */
     int_t nprow;          /* number of process rows */
     int_t npcol;          /* number of process columns */
@@ -595,7 +596,7 @@ typedef struct
     void *next;
 } etree_node;
 
-struct pair
+struct superlu_pair
 {
     int ind;
     int val;
@@ -657,6 +658,11 @@ extern void    superlu_free_dist (void*);
 extern int_t   *intMalloc_dist (int_t);
 extern int_t   *intCalloc_dist (int_t);
 extern int_t   mc64id_dist(int_t *);
+extern void  arrive_at_ublock (int_t, int_t *, int_t *, int_t *,
+			       int_t *, int_t *, int_t, int_t, 
+			       int_t *, int_t *, int_t *, gridinfo_t *);
+extern int_t estimate_bigu_size (int_t, int_t, int_t **, Glu_persist_t *,
+				 gridinfo_t *, int_t *);
 
 /* Auxiliary routines */
 extern double SuperLU_timer_ ();
@@ -713,11 +719,11 @@ extern int_t get_cublas_nb ();
 extern int_t get_num_cuda_streams ();
 #endif
 
-extern int compare_pair (const void *, const void *);
 extern int get_thread_per_process();
 extern int_t get_max_buffer_size ();
 extern int_t get_min (int_t *, int_t);
-extern int_t static_partition (struct pair *, int_t, int_t *, int_t,
+extern int compare_pair (const void *, const void *);
+extern int_t static_partition (struct superlu_pair *, int_t, int_t *, int_t,
 			       int_t *, int_t *, int);
 
 /* Routines for debugging */
